@@ -1,5 +1,10 @@
+#include "config.h"
 #include <libnjb.h>
+#ifdef HAVE_GETOPT_H
 #include <getopt.h>
+#else
+#include <unistd.h>
+#endif
 
 void usage (void);
 
@@ -37,8 +42,10 @@ int main (int argc, char **argv)
 	if ( debug ) NJB_Set_Debug(debug);
 
 	if ( NJB_Discover(njbs, 0, &n) == -1 ) {
-		njb_error_dump(stderr);
+	        fprintf(stderr, "could not locate any jukeboxes\n");
+		return 1;
 	}
+
 	if ( n == 0 ) {
 		fprintf(stderr, "no NJB devices found\n");
 		return 0;
@@ -47,17 +54,17 @@ int main (int argc, char **argv)
 	njb= njbs;
 
 	if ( NJB_Open(njb) == -1 ) {
-		njb_error_dump(stderr);
+		NJB_Error_Dump(njb, stderr);
 		return 1;
 	}
 
 	if ( NJB_Capture(njb) == -1 ) {
-		njb_error_dump(stderr);
+		NJB_Error_Dump(njb, stderr);
 		return 1;
 	}
 
 	if ( NJB_Delete_Track(njb, id) == -1 ) {
-		njb_error_dump(stderr);
+		NJB_Error_Dump(njb, stderr);
 	}
 
 	NJB_Release(njb);
