@@ -1,11 +1,14 @@
+#include "config.h"
 #include <libnjb.h>
+#ifdef HAVE_GETOPT_H
 #include <getopt.h>
+#else
+#include <unistd.h>
+#endif
 
 int progress(u_int64_t sent, u_int64_t total, const char* buf, unsigned len, void *data );
 void usage(void);
 
-extern int njb_error;
-
 int main (int argc, char **argv)
 {
 	njb_t njbs[NJB_MAX_DEVICES], *njb;
@@ -37,15 +40,15 @@ int main (int argc, char **argv)
 			return 1;
 		}
 	}
-	argc-= optind;
-	argv+= optind;
+	argc -= optind;
+	argv += optind;
 
 	if ( argc != 2 ) {
 		usage();
 		return 1;
 	}
 
-	id= strtoul(argv[0], &endptr, 10);
+	id = strtoul(argv[0], &endptr, 10);
 	if ( *endptr != 0 ) {
 		fprintf(stderr, "illegal value %s\n", optarg);
 		return 1;
@@ -54,40 +57,44 @@ int main (int argc, char **argv)
 		return 1;
 	}
 
-	file= argv[1];
+	file = argv[1];
 
 	if ( debug ) NJB_Set_Debug(debug);
 
-	if ( NJB_Discover(njbs, 0, &n) == -1 ) {
-		njb_error_dump(stderr);
-		return 1;
+
+	if (NJB_Discover(njbs, 0, &n) == -1) {
+	  fprintf(stderr, "could not locate any jukeboxes\n");
+	  return 1;
 	}
+
 	if ( n == 0 ) {
 		fprintf(stderr, "no NJB devices found\n");
 		return 0;
 	} 
 
-	njb= njbs;
+	njb = njbs;
 
 	if ( NJB_Open(njb) == -1 ) {
-		njb_error_dump(stderr);
+		NJB_Error_Dump(njb, stderr);
 		return 1;
 	}
 
 	NJB_Capture(njb);
 
 	if ( ! size ) {
-		datafile_t *df;
+		njb_datafile_t *df;
 
 		printf("Locating file %u\n", id);
 		NJB_Reset_Get_Datafile_Tag(njb);
-		while ( (df= NJB_Get_Datafile_Tag(njb)) ) {
+		while ( (df = NJB_Get_Datafile_Tag(njb)) ) {
 			if ( df->dfid == id ) {
-				size= datafile_size(df);
+				size = df->filesize;
 			}
-			datafile_destroy(df);
+			NJB_Datafile_Destroy(df);
 		}
-		if ( njb_error != EO_EOM ) njb_error_dump(stderr);
+
+		/* Dump any pending errors */
+		NJB_Error_Dump(njb, stderr);
 
 		if ( size ) {
 			printf("%Lu bytes\n", size);
@@ -98,7 +105,7 @@ int main (int argc, char **argv)
 
 	if ( size ) {
 		if ( NJB_Get_File(njb, id, (u_int32_t) size, file, progress, NULL ) == -1 ) {
-			njb_error_dump(stderr);
+			NJB_Error_Dump(njb, stderr);
 		}
 		printf("\n");
 	}
