@@ -1,6 +1,11 @@
+#include "config.h"
 #include <string.h>
 #include <libnjb.h>
+#ifdef HAVE_GETOPT_H
 #include <getopt.h>
+#else
+#include <unistd.h>
+#endif
 
 int main (int argc, char **argv)
 {
@@ -39,8 +44,9 @@ int main (int argc, char **argv)
 	  }
 	}
 
-	if ( NJB_Discover(njbs, 0, &n) == -1 ) {
-		njb_error_dump(stderr);
+	if (NJB_Discover(njbs, 0, &n) == -1) {
+	  fprintf(stderr, "could not locate any jukeboxes\n");
+	  return 1;
 	}
 
 	if ( n == 0 ) {
@@ -48,15 +54,20 @@ int main (int argc, char **argv)
 		return 1;
 	}
 
-	njb= njbs;
+	njb = njbs;
 	if ( NJB_Open(njb) == -1 ) {
-		njb_error_dump(stderr);
+		NJB_Error_Dump(njb, stderr);
 		return 1;
 	}
 
 	ownerstring = NJB_Get_Owner_String (njb);
-	printf("Owner string: %s\n", ownerstring);
-	free(ownerstring);
+	if (ownerstring != NULL) {
+	  printf("Owner string: %s\n", ownerstring);
+	  free(ownerstring);
+	} else {
+	  fprintf(stderr, "could not retrieve owner string\n");
+	  NJB_Error_Dump(njb, stderr);
+	}
 
 	NJB_Close(njb);
 	return 0;
