@@ -1,3 +1,11 @@
+/**
+ * \file protocol.c
+ *
+ * This file contains the major parts of the NJB1 (OaSIS) protocol
+ * for all jukebox functionality.
+ */
+
+#include "config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <unistd.h>
@@ -14,6 +22,10 @@
 #include "defs.h"
 #include "base.h"
 #include "eax.h"
+#include "songid.h"
+#include "datafile.h"
+#include "njbtime.h"
+#include "playlist.h"
 
 extern int __sub_depth;
 
@@ -21,6 +33,9 @@ extern int __sub_depth;
 /**
  * Initializes the basic state of the njb->protocol_state for the
  * NJB1-device.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @return 0 on success, -1 on failure.
  */
 int njb_init_state (njb_t *njb) {
   __dsub= "njb_init_state";
@@ -42,7 +57,9 @@ int njb_init_state (njb_t *njb) {
   state->reset_get_track_tag = 0;
   state->reset_get_playlist = 0;
   state->reset_get_datafile_tag = 0;
+  state->power = NJB_POWER_BATTERY;
   njb->protocol_state = (unsigned char *) state;
+  memset(&state->sdmiid, 0, 16);
 
   __leave;
   return 0;
@@ -57,6 +74,133 @@ int njb_init_state (njb_t *njb) {
  * what it means, but I have implemented it where I've run into it.
  */
 
+
+/**
+ * This returns the string representing an internal NJB1 error code.
+ */
+#define NJB_STATUS(a,b) { \
+	char *s = njb_status_string(b);\
+	njb_error_add_string(a,subroutinename,s);\
+	free (s); \
+}
+static char *njb_status_string (unsigned char code)
+{
+  char buffer[100];
+  const char *status = NULL;
+  const char *fmt = "%s";
+ 
+  switch (code)
+    {
+    case NJB_MSG_OKAY:
+      status = "no error";
+      break;
+    case NJB_ERR_FAILED:
+      status = "operation failed";
+      break;
+    case NJB_ERR_DEVICE_BUSY:
+      status = "device busy";
+      break;
+    case NJB_ERR_STORAGE_FULL:
+      status = "storage full";
+      break;
+    case NJB_ERR_HD_GENERAL_ERROR:
+      status = "general hard drive failure";
+      break;
+    case NJB_ERR_SETTIME_REJECTED:
+      status = "set time rejected";
+      break;
+      
+    case NJB_ERR_TRACK_NOT_FOUND:
+      status = "track not found";
+      break;
+    case NJB_ERR_TRACK_EXISTS:
+      status = "track exists";
+      break;
+    case NJB_ERR_TITLE_MISSING:
+      status = "title missing";
+      break;
+    case NJB_ERR_CODEC_MISSING:
+      status = "CODEC missing";
+      break;
+    case NJB_ERR_SIZE_MISSING:
+      status = "size missing";
+      break;
+    case NJB_ERR_IO_OPERATION_ABORTED:
+      status = "I/O operation aborted";
+      break;
+    case NJB_ERR_READ_WRITE_ERROR:
+      status = "read/write error";
+      break;
+    case NJB_ERR_NOT_OPENED:
+      status = "not opened";
+      break;
+    case NJB_ERR_UPLOAD_DENIED:
+      status = "upload denied";
+      break;
+      
+    case NJB_ERR_PLAYLIST_NOT_FOUND:
+      status = "playlist not found";
+      break;
+    case NJB_ERR_PLAYLIST_EXISTS:
+      status = "playlist exists";
+      break;
+    case NJB_ERR_PLAYLIST_ITEM_NOT_FOUND:
+      status = "playlist item not found";
+      break;
+    case NJB_ERR_PLAYLIST_ITEM_EXISTS:
+      status = "playlist item exists";
+      break;
+      
+    case NJB_MSG_QUEUED_AUDIO_STARTED:
+      status = "queued audio started";
+      break;
+    case NJB_MSG_PLAYER_FINISHED:
+      status = "player finished";
+      break;
+
+    case NJB_ERR_QUEUE_ALREADY_EMPTY:
+      status = "queue already empty";
+      break;
+    case NJB_ERR_VOLUME_CONTROL_UNAVAILABLE:
+      status = "volume control unavailable";
+      break;
+      
+    case NJB_ERR_DATA_NOT_FOUND:
+      status = "data not found";
+      break;
+    case NJB_ERR_DATA_NOT_OPENED:
+      status = "data not opened";
+      break;
+      
+    case NJB_ERR_UNDEFINED_ERR:
+      status = "undefined error";
+      break;
+      /*
+	This is currently not used and does not have a meaningful value
+	case NJB_ERR_NOT_IMPLEMENTED:
+	status = "function not implemented";
+	break;
+      */
+    default:
+      status = "unknown error";
+      fmt = "%s %02x";
+      break;
+    }
+  
+  sprintf (buffer, fmt, status, code);
+  
+  return strdup (buffer);
+}
+
+/**
+ * This function sets the library counter on the device.
+ * The library counter is sent back and forth to the device to indicate if
+ * something in the library has changed.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @param count is the current library counter to set on the device.
+ * @return 0 on success, -1 on failure.
+ */
 int njb_set_library_counter (njb_t *njb, u_int64_t count)
 {
 	__dsub= "njb_set_library_counter";
@@ -70,7 +214,7 @@ int njb_set_library_counter (njb_t *njb, u_int64_t count)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_SET_LIBRARY_COUNTER, 0, 0, 8, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -82,6 +226,15 @@ int njb_set_library_counter (njb_t *njb, u_int64_t count)
 	return 0;
 }
 
+/**
+ * This function gets the library counter from the device.
+ * The library counter is sent back and forth to the device to indicate if
+ * something in the library as changed.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @lcount is a pointer to the current library counter to get from the device.
+ * @return 0 on success, -1 on failure.
+ */
 int njb_get_library_counter (njb_t *njb, njblibctr_t *lcount)
 {
 	__dsub= "njb_get_library_counter";
@@ -94,13 +247,13 @@ int njb_get_library_counter (njb_t *njb, njblibctr_t *lcount)
 	
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_GET_LIBRARY_COUNTER, 0, 0, 25, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] & 0xf ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	} else if ( data[0] ) {
@@ -119,60 +272,69 @@ int njb_get_library_counter (njb_t *njb, njblibctr_t *lcount)
 	return 0;
 }
 
-int njb_ping (njb_t *njb, njbid_t *njbid)
+/**
+ * This function pings the NJB1 device to see if it is up and running,
+ * and ready for action. This is done every now and then.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @return 0 on success, -1 on failure.
+ */
+int njb_ping (njb_t *njb)
 {
-	__dsub= "njb_ping";
-	ssize_t bread;
-	unsigned char data[58];
-
-	__enter;
-	
-	memset(data, 0, 58);
-
-       	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
-       		NJB_CMD_PING, 0, 0, 0, NULL) == -1 ) {
-       		NJB_ERROR(EO_USBCTL);
-       		__leave;
-       		return -1;
-       	}
-
-       	if ( (bread= usb_pipe_read(njb, data, 58)) == -1 ) {
-       		NJB_ERROR(EO_USBBLK);
-       		__leave;
-       		return -1;
-       	} else if ( bread < 58 ) {
-       		NJB_ERROR(EO_RDSHORT);
-       		__leave;
-       	       	return -1;
-       	}
-
-       	if ( data[0] ) {
-       		NJB_STATUS(data[0]);
-       		__leave;
-       		return -1;
-       	}
-       	memcpy(&njbid->id, &data[1], 16);
-       	njbid->fwRel= (u_int8_t) 0;
-       	njbid->fwMinor= (u_int8_t) data[19];
-       	njbid->fwMajor= (u_int8_t) data[20];
-	/* Copy this -- eventually move it into njb_t */
-	njb->fwRel = njbid->fwRel;
-	njb->fwMinor = njbid->fwMinor;
-	njb->fwMajor = njbid->fwMajor;
-       	memcpy(&njbid->productName, &data[25], 32);
-       	njbid->power= (u_int8_t) data[57];
+  __dsub= "njb_ping";
+  ssize_t bread;
+  unsigned char data[58];
+  njb_state_t *state = (njb_state_t *) njb->protocol_state;
+  
+  __enter;
+  
+  memset(data, 0, 58);
+  
+  if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
+		 NJB_CMD_PING, 0, 0, 0, NULL) == -1 ) {
+    NJB_ERROR(njb, EO_USBCTL);
+    __leave;
+    return -1;
+  }
+  
+  if ( (bread= usb_pipe_read(njb, data, 58)) == -1 ) {
+    NJB_ERROR(njb, EO_USBBLK);
+    __leave;
+    return -1;
+  } else if ( bread < 58 ) {
+    NJB_ERROR(njb, EO_RDSHORT);
+    __leave;
+    return -1;
+  }
 
+  if ( data[0] ) {
+    NJB_STATUS(njb,data[0]);
+    __leave;
+    return -1;
+  }
+  memcpy(&state->sdmiid, &data[1], 16);
+  state->fwMinor = (u_int8_t) data[19];
+  state->fwMajor = (u_int8_t) data[20];
+  memcpy(&state->productName, &data[25], 32);
+  state->power= (u_int8_t) data[57];
+  
 #if (DEBUG>=2)
-	printf("NJB ID   : ");
-	data_dump(stdout, &njbid->id, 16);
-	printf("Firmware : %d.%d\n", njbid->fwMajor, njbid->fwMinor);
-	printf("Product  : %s\n", njbid->productName);
-	printf("Power    : %d\n", njbid->power);
+  printf("NJB ID   : ");
+  data_dump(stdout, &state->sdmiid, 16);
+  printf("Firmware : %d.%d\n", state->fwMajor, state->fwMinor);
+  printf("Product  : %s\n", state->productName);
+  printf("Power    : %d\n", state->power);
 #endif
-	__leave;
-	return 0;
+  __leave;
+  return 0;
 }
 
+/**
+ * This function verifies if the last command was successful or not.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @return 0 on success, -1 on failure.
+ */
 int njb_verify_last_command (njb_t *njb)
 {
 	__dsub= "njb_verify_last_command";
@@ -183,13 +345,13 @@ int njb_verify_last_command (njb_t *njb)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_VERIFY_LAST_CMD, 0, 0, 1, &data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -198,6 +360,12 @@ int njb_verify_last_command (njb_t *njb)
 	return 0;
 }
 
+/**
+ * This function captures the NJB1 device.
+ *
+ * @param njb is a pointer to the jukebox object to use.
+ * @return 0 on success, -1 on failure.
+ */
 int njb_capture (njb_t *njb, int which)
 {
 	__dsub= "njb_capture";
@@ -211,13 +379,13 @@ int njb_capture (njb_t *njb, int which)
        	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
        		mode, 0, 0, 1, &data) == -1 ) {
 
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
        		__leave;
        		return -1;
        	}
 
        	if ( data && (data & 0xf) ) {
-       		NJB_ERROR(EO_BADSTATUS);
+       		NJB_ERROR(njb, EO_BADSTATUS);
        		__leave;
        		return -1;
        	}
@@ -226,6 +394,12 @@ int njb_capture (njb_t *njb, int which)
 	return 0;
 }
 
+/**
+ * Get a track header from the NJB1.
+ *
+ * @return 0 on OK, -1 on error, -2 if the last header has been
+ *         retrieved.
+ */
 int njb_get_track_tag_header (njb_t *njb, njbttaghdr_t *tagh, int cmd)
 {
 	__dsub= "njb_get_track_tag_header";
@@ -238,17 +412,21 @@ int njb_get_track_tag_header (njb_t *njb, njbttaghdr_t *tagh, int cmd)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
 		 == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] == NJB_ERR_TRACK_NOT_FOUND ) {
-		NJB_ERROR(EO_EOM);
+	        /*
+	         * Ignore this "error" as of now, it probably
+		 * just mean "end of transmission" or "zero tags left". 
+		 */
+	        /* NJB_ERROR(njb, EO_EOM); return -1; */
 		__leave;
-		return -1;
+		return -2;
 	} else if ( data[0] ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -260,20 +438,20 @@ int njb_get_track_tag_header (njb_t *njb, njbttaghdr_t *tagh, int cmd)
 	return 0;
 }
 
-songid_t *njb_get_track_tag (njb_t *njb, njbttaghdr_t *tagh)
+njb_songid_t *njb_get_track_tag (njb_t *njb, njbttaghdr_t *tagh)
 {
 	__dsub= "njb_get_track_tag";
 	u_int16_t msw, lsw;
 	ssize_t bread;
 	void *data;
-	songid_t *song;
+	njb_songid_t *song;
 	unsigned char *dp;
 
 	__enter;
 
 	data= malloc(tagh->size+5);
 	if ( data == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return NULL;
 	}
@@ -284,14 +462,14 @@ songid_t *njb_get_track_tag (njb_t *njb, njbttaghdr_t *tagh)
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_GET_TRACK_TAG, msw, lsw, 0, NULL) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return NULL;
 	}
 
 	if ( (bread= usb_pipe_read(njb, data, tagh->size+5)) == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(data);
 		__leave;
 		return NULL;
@@ -308,6 +486,12 @@ songid_t *njb_get_track_tag (njb_t *njb, njbttaghdr_t *tagh)
 	return song;
 }
 
+/**
+ * Gets a playlist header.
+ *
+ * @return 0 on OK, -1 on error, -2 means "retry", -3 means
+ *         "not found" or "last playlist already retrieved".
+ */
 int njb_get_playlist_header (njb_t *njb, njbplhdr_t *plh, int cmd)
 {
 	__dsub= "njb_get_playlist_header";
@@ -320,23 +504,26 @@ int njb_get_playlist_header (njb_t *njb, njbplhdr_t *plh, int cmd)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
 		 == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] == NJB_ERR_PLAYLIST_NOT_FOUND ) {
-		NJB_ERROR(EO_EOM);
+	        /*
+	         * Ignore this "error" as of now, it probably
+		 * just mean "end of transmission" or "zero tags left". 
+		 */
+	        /* NJB_ERROR(njb, EO_EOM); return -1; */
 		__leave;
-		return -1;
+		return -3;
 	} else if ( (data[0]) & 0xf ) { 
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	} else if ( data[0] ) {
-		NJB_ERROR(EO_AGAIN);
 		__leave;
-		return -1;
+		return -2;
 	}
 
 	plh->plid = njb1_bytes_to_32bit(&data[1]);
@@ -346,20 +533,20 @@ int njb_get_playlist_header (njb_t *njb, njbplhdr_t *plh, int cmd)
 	return 0;
 }
 
-playlist_t *njb_get_playlist (njb_t *njb, njbplhdr_t *plh)
+njb_playlist_t *njb_get_playlist (njb_t *njb, njbplhdr_t *plh)
 {
 	__dsub= "njb_get_playlist";
 	u_int16_t msw, lsw;
 	ssize_t bread;
 	void *data;
-	playlist_t *pl;
+	njb_playlist_t *pl;
 	unsigned char *dp;
 
 	__enter;
 
-	data= malloc(plh->size+5);
+	data = malloc(plh->size+5);
 	if ( data == NULL ) {
-		NJB_ERROR(EO_EOM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return NULL;
 	}
@@ -369,29 +556,28 @@ playlist_t *njb_get_playlist (njb_t *njb, njbplhdr_t *plh)
 	msw= get_msw(plh->plid);
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
-		NJB_CMD_GET_PLAYLIST, msw, lsw, 0, NULL) == -1 ) {
-
-		NJB_ERROR(EO_USBCTL);
+	     NJB_CMD_GET_PLAYLIST, msw, lsw, 0, NULL) == -1 ) {
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return NULL;
 	}
 
 	if ( (bread= usb_pipe_read(njb, data, plh->size+5)) == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(data);
 		__leave;
 		return NULL;
 	} else if ( bread < plh->size+5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		free(data);
 		__leave;
 		return NULL;
 	}
 
-	dp= (unsigned char *) data;
+	dp = (unsigned char *) data;
 
-	pl= playlist_unpack(&dp[5], plh->size);
+	pl = playlist_unpack(&dp[5], plh->size);
 
 	free(data);
 	
@@ -399,7 +585,12 @@ playlist_t *njb_get_playlist (njb_t *njb, njbplhdr_t *plh)
 	return pl;
 }
 
-int njb_get_disk_usage (njb_t *njb, u_int64_t *total, u_int64_t *free)
+/**
+ * Get disk usage for the NJB1
+ *
+ * @return 0 on OK, -1 on failure, -2 means "try again"
+ */
+int njb_get_disk_usage (njb_t *njb, u_int64_t *total, u_int64_t *free_bytes)
 {
 	__dsub= "njb_get_disk_usage";
 	unsigned char data[17];
@@ -411,23 +602,22 @@ int njb_get_disk_usage (njb_t *njb, u_int64_t *total, u_int64_t *free)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_GET_DISK_USAGE, 0, 0, 17, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] & 0xf ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	} else if ( data[0] ) {
-		NJB_ERROR(EO_AGAIN);
 		__leave;
-		return -1;
+		return -2;
 	}
 
 	*total = njb1_bytes_to_64bit(&data[1]);
-	*free = njb1_bytes_to_64bit(&data[9]);
+	*free_bytes = njb1_bytes_to_64bit(&data[9]);
 
 	__leave;
 	return 0;
@@ -445,7 +635,7 @@ int njb_get_owner_string (njb_t *njb, owner_string name)
        	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_GET_OWNER_STRING, 0, 0, 0, NULL) == -1 ) {
 
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
        		__leave;
        		return -1;
        	}
@@ -453,21 +643,21 @@ int njb_get_owner_string (njb_t *njb, owner_string name)
        	if ( (bread= usb_pipe_read(njb, data, OWNER_STRING_LENGTH+1))
        		== -1 ) {
 
-       		NJB_ERROR(EO_USBBLK);
+       		NJB_ERROR(njb, EO_USBBLK);
        		__leave;
        		return -1;
        	} else if ( bread < OWNER_STRING_LENGTH+1 ) {
-       		NJB_ERROR(EO_RDSHORT);
+       		NJB_ERROR(njb, EO_RDSHORT);
        		__leave;
        		return -1;
        	}
        	if ( data[0] ) {
-       		NJB_STATUS((unsigned char) data[0]);
+       		NJB_STATUS(njb,(unsigned char) data[0]);
        		__leave;
        		return -1;
        	}
 
-       	strncpy(name, &data[1], OWNER_STRING_LENGTH);
+       	strncpy((char *) name, &data[1], OWNER_STRING_LENGTH);
 	name[OWNER_STRING_LENGTH]= '\0'; /* Just in case */
 
 	__leave;
@@ -484,7 +674,7 @@ int njb_set_owner_string (njb_t *njb, owner_string name)
        	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_SET_OWNER_STRING, 0, 0, 0, NULL) == -1 ) {
 
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
        		__leave;
        		return -1;
        	}
@@ -492,11 +682,11 @@ int njb_set_owner_string (njb_t *njb, owner_string name)
        	if ( (bwritten= usb_pipe_write(njb, name, OWNER_STRING_LENGTH))
        		== -1 ) {
 
-       		NJB_ERROR(EO_USBBLK);
+       		NJB_ERROR(njb, EO_USBBLK);
        		__leave;
        		return -1;
        	} else if ( bwritten < OWNER_STRING_LENGTH ) {
-       		NJB_ERROR(EO_WRSHORT);
+       		NJB_ERROR(njb, EO_WRSHORT);
        		__leave;
        		return -1;
        	}
@@ -506,6 +696,12 @@ int njb_set_owner_string (njb_t *njb, owner_string name)
 	return 0;
 }
 
+/**
+ * Retrieves a datafile header from the NJB1
+ *
+ * @return 0 on OK, -1 on error, -2 means the last header
+ *         has already been returned.
+ */
 int njb_get_datafile_header (njb_t *njb, njbdfhdr_t *dfh, int cmd)
 {
 	__dsub= "njb_get_datafile_header";
@@ -518,17 +714,21 @@ int njb_get_datafile_header (njb_t *njb, njbdfhdr_t *dfh, int cmd)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER, cmd, 0, 0, 9, data)
 		 == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return 0;
 	}
 
-	if ( data[0] == 0x60 ) {
-		NJB_ERROR(EO_EOM);
+	if ( data[0] == NJB_ERR_DATA_NOT_FOUND ) {
+	        /*
+	         * Ignore this "error" as of now, it probably
+		 * just mean "end of transmission" or "zero tags left". 
+		 */
+	        /* NJB_ERROR(njb, EO_EOM); return -1; */
 		__leave;
-		return -1;
+		return -2;
 	} else if ( data[0] ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -540,20 +740,20 @@ int njb_get_datafile_header (njb_t *njb, njbdfhdr_t *dfh, int cmd)
 	return 0;
 }
 
-datafile_t *njb_get_datafile_tag (njb_t *njb, njbdfhdr_t *dfh)
+njb_datafile_t *njb_get_datafile_tag (njb_t *njb, njbdfhdr_t *dfh)
 {
 	__dsub= "njb_get_datafile_tag";
 	u_int16_t msw, lsw;
 	ssize_t bread;
 	void *data;
-	datafile_t *df;
+	njb_datafile_t *df;
 	unsigned char *dp;
 
 	__enter;
 
 	data= malloc(dfh->size+5);
 	if ( data == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return NULL;
 	}
@@ -565,19 +765,19 @@ datafile_t *njb_get_datafile_tag (njb_t *njb, njbdfhdr_t *dfh)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_GET_DATAFILE_TAG, msw, lsw, 0, NULL) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return NULL;
 	}
 
 	if ( (bread= usb_pipe_read(njb, data, dfh->size+5)) == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(data);
 		__leave;
 		return NULL;
 	} else if ( bread < dfh->size+5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		free(data);
 		__leave;
 		return NULL;
@@ -618,7 +818,7 @@ u_int32_t njb_receive_file_block (njb_t *njb, u_int32_t offset, u_int32_t bsize,
 
 	/* Too large transfer requested */
 	if ( bsize > NJB_XFER_BLOCK_SIZE ) {
-		NJB_ERROR(EO_TOOBIG);
+		NJB_ERROR(njb, EO_TOOBIG);
 		__leave;
 		return -1;
 	}
@@ -634,14 +834,14 @@ u_int32_t njb_receive_file_block (njb_t *njb, u_int32_t offset, u_int32_t bsize,
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		       NJB_CMD_RECEIVE_FILE_BLOCK, 0, 0, 8, filedata) == -1 ) {
 	  
-	  NJB_ERROR(EO_USBCTL);
+	  NJB_ERROR(njb, EO_USBCTL);
 	  __leave;
 	  return -1;
 	}
 
 	/* Read the status and block header first */
 	if ( (bread = usb_pipe_read(njb, bp, bsize + NJB_XFER_BLOCK_HEADER_SIZE)) == -1 ) {
-	  NJB_ERROR(EO_USBBLK);
+	  NJB_ERROR(njb, EO_USBBLK);
 	  __leave;
 	  return -1;
 	} else if ( bread < bsize + NJB_XFER_BLOCK_HEADER_SIZE ) {
@@ -652,7 +852,7 @@ u_int32_t njb_receive_file_block (njb_t *njb, u_int32_t offset, u_int32_t bsize,
 	/* Check status in header, should it be if (!lastshort)??? */
 	status = ((unsigned char *)bp)[0];
 	if (status) {
-	  NJB_STATUS(status);
+	  NJB_STATUS(njb,status);
 	  __leave;
 	  return -1;
 	}
@@ -673,7 +873,7 @@ int njb_request_file (njb_t *njb, u_int32_t fileid)
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_REQUEST_TRACK, 1, 0, 4, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -692,13 +892,13 @@ int njb_transfer_complete (njb_t *njb)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_TRANSFER_COMPLETE, 0, 0, 1, &data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data && (data & 0xf) ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -724,35 +924,35 @@ int njb_send_track_tag (njb_t *njb, njbttaghdr_t *tagh, void *tag)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_SEND_TRACK_TAG, 0, 0, 4, utagsz) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
-	bwritten= usb_pipe_write(njb, tag, tagh->size);
+	bwritten = usb_pipe_write(njb, tag, tagh->size);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bwritten < tagh->size ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		__leave;
 		return -1;
 	}
 
-	bread= usb_pipe_read(njb, data, 5);
+	bread = usb_pipe_read(njb, data, 5);
 	if ( bread == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bread < 5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -782,7 +982,7 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_SEND_DATAFILE_TAG, 0, 0, 4, utagsz) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -792,7 +992,7 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 
 	padded_tag = (unsigned char *) malloc(dfh->size+5);
 	if ( padded_tag == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
@@ -805,12 +1005,12 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 	bwritten= usb_pipe_write(njb, padded_tag, dfh->size+5);
 	if ( bwritten == -1 ) {
 		free(padded_tag);
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bwritten < dfh->size+5 ) {
 		free(padded_tag);
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		__leave;
 		return -1;
 	}
@@ -821,17 +1021,17 @@ int njb_send_datafile_tag (njb_t *njb, njbdfhdr_t *dfh, void *tag)
 
 	bread = usb_pipe_read(njb, data, 5);
 	if ( bread == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bread < 5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] ) {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -861,7 +1061,7 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 	__enter;
 
 	if ( blocksize > NJB_XFER_BLOCK_SIZE ) {
-		NJB_ERROR(EO_TOOBIG);
+		NJB_ERROR(njb, EO_TOOBIG);
 		__leave;
 		return -1;
 	}
@@ -879,7 +1079,7 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 			 NJB_CMD_SEND_FILE_BLOCK, lsw, msw, 1, &status)
 	       == -1 ) {
 	    
-	    NJB_ERROR(EO_USBCTL);
+	    NJB_ERROR(njb, EO_USBCTL);
 	    __leave;
 	    return -1;
 	  }
@@ -896,12 +1096,20 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 	     * Generally, the nomad needs to wait 200-800 ms. */
 #ifdef HAVE_USLEEP
 	    usleep(200000);
+#else
+#ifdef __WIN32__
+	    /* 
+	     * This is because sleep() is actually deprecated in favor of
+	     * Sleep() on Windows.
+	     */
+	    _sleep(1);
 #else
 	    sleep(1);
+#endif
 #endif
 	    retries --;
 	    if ( !retries ) {
-	      NJB_ERROR(EO_BADSTATUS);
+	      NJB_ERROR(njb, EO_BADSTATUS);
 	      __leave;
 	      return -1;
 	    }
@@ -911,7 +1119,7 @@ u_int32_t njb_send_file_block (njb_t *njb, void *data, u_int32_t blocksize)
 	bwritten = usb_pipe_write(njb, data, blocksize);
 
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	}
@@ -930,13 +1138,13 @@ int njb_stop_play (njb_t *njb)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_STOP_PLAY, 0, 0, 1, &data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -957,13 +1165,13 @@ int njb_get_eax_size (njb_t *njb, u_int32_t *size)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_GET_EAX_SIZE, 0, 0, 5, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data[0] ) {
-		NJB_ERROR(EO_BADSTATUS);
+		NJB_ERROR(njb, EO_BADSTATUS);
 		__leave;
 		return -1;
 	}
@@ -974,71 +1182,15 @@ int njb_get_eax_size (njb_t *njb, u_int32_t *size)
 	return 0;
 }
 
-/* This is an old, deprecated routine */
-eax_t *njb_get_eax (njb_t *njb, u_int32_t size)
-{
-	__dsub= "njb_get_eax";
-	eax_t *eax= NULL;
-	char *data;
-	ssize_t bread;
-	u_int32_t actsize;
-	unsigned char usize[4];
-
-	__enter;
-
-	data= (char *) malloc(size);
-	if ( data == NULL ) {
-		NJB_ERROR(EO_NOMEM);
-		__leave;
-		return NULL;
-	}
-	memset(data, 0, size);
-	memset(usize, 0, 4);
-	
-	from_32bit_to_njb1_bytes(size, &usize[0]);
-
-	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER, NJB_CMD_GET_EAX,
-		0, 0, 4, usize) == -1 ) {
-
-		NJB_ERROR(EO_USBCTL);
-		free(data);
-		__leave;
-		return NULL;
-	}
-
-	bread= usb_pipe_read(njb, data, size+5);
-	if ( bread == -1 ) {
-		NJB_ERROR(EO_USBBLK);
-		free(data);
-		__leave;
-		return NULL;
-	} else if ( bread < size ) {
-		NJB_ERROR(EO_RDSHORT);
-		free(data);
-		__leave;
-		return NULL;
-	}
-
-	if ( data[0] ) {
-		NJB_STATUS((int) data[0]);
-		free(data);
-		__leave;
-		return NULL;
-	}
-
-	actsize = njb1_bytes_to_32bit(&data[1]);
-
-	/* Unpack the EAX structure */
-
-	eax= eax_unpack(&data[5], actsize);
-
-	free(data);
-
-	__leave;
-	return eax;
-}
-
-/* This is the new EAX API */
+/**
+ * This reads in the block with EAX types. It is a byte
+ * chunk which is then interpreted and the EAX effects are
+ * added to a list of effects which is then retrieved by
+ * calls to the <code>njb_get_nexteax()</code> function.
+ * 
+ * @param njb a pointer to the NJB object to use
+ * @param size the previously determined size of the EAX block
+ */
 void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
 {
   __dsub= "njb_read_eaxtypes";
@@ -1048,12 +1200,11 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
   ssize_t bread;
   u_int32_t actsize;
   unsigned char usize[4];
-
   __enter;
 
   data = (unsigned char *) malloc(size);
   if ( data == NULL ) {
-    NJB_ERROR(EO_NOMEM);
+    NJB_ERROR(njb, EO_NOMEM);
     __leave;
     return;
   }
@@ -1065,7 +1216,7 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
   if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER, NJB_CMD_GET_EAX,
 		 0, 0, 4, usize) == -1 ) {
     
-    NJB_ERROR(EO_USBCTL);
+    NJB_ERROR(njb, EO_USBCTL);
     free(data);
     __leave;
     return;
@@ -1073,19 +1224,19 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
 
   bread = usb_pipe_read(njb, data, size+5);
   if ( bread == -1 ) {
-    NJB_ERROR(EO_USBBLK);
+    NJB_ERROR(njb, EO_USBBLK);
     free(data);
     __leave;
     return;
   } else if ( bread < size ) {
-    NJB_ERROR(EO_RDSHORT);
+    NJB_ERROR(njb, EO_RDSHORT);
     free(data);
     __leave;
     return;
   }
 
   if ( data[0] ) {
-    NJB_STATUS((int) data[0]);
+    NJB_STATUS(njb,(int) data[0]);
     free(data);
     __leave;
     return;
@@ -1094,7 +1245,7 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
   actsize = njb1_bytes_to_32bit(&data[1]);
 
   /* Unpack the EAX structure */
-  eax_unpack_new_api(&data[5], actsize, state);
+  eax_unpack(&data[5], actsize, state);
 
   free(data);
 
@@ -1102,6 +1253,12 @@ void njb_read_eaxtypes (njb_t *njb, u_int32_t size)
   return;
 }
 
+/**
+ * This retrieves an EAX effect from the list of
+ * effects read in by the previous functions.
+ *
+ * @param njb a pointer to the NJB object to use
+ */
 njb_eax_t *njb_get_nexteax(njb_t *njb)
 {
   njb_state_t *state = (njb_state_t *) njb->protocol_state;
@@ -1110,12 +1267,25 @@ njb_eax_t *njb_get_nexteax(njb_t *njb)
   ret = state->next_eax;
   if (ret != NULL) {
     state->next_eax = ret->next;
+  } else {
+    /* 
+     * When we have returned the last EAX type, make sure
+     * there are no dangerous pointers into the void
+     */
+    state->first_eax = NULL;
   }
   return ret;
 }
 
-/* This routine is dangerous. Do not call it... */
-int njb_refresh_eax(njb_t *njb, eax_t *eax)
+/*
+ * This routine is dangerous. At one time, the NJB1 would
+ * use this to update the status of the EAX settings, but
+ * with newer firmware it has become increasingly unreliable.
+ * So we simply do not use it, re-read the EAX types instead. 
+ */
+
+/*
+int njb_refresh_eax(njb_t *njb)
 {
 	__dsub= "njb_refresh_eax";
 	unsigned char *data;
@@ -1126,7 +1296,7 @@ int njb_refresh_eax(njb_t *njb, eax_t *eax)
 
 	data= malloc(4);
 	if (data == NULL) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
@@ -1134,42 +1304,42 @@ int njb_refresh_eax(njb_t *njb, eax_t *eax)
 	
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 			NJB_CMD_GET_EAX, 1, 0, 4, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return -1;
 	}
 
-	/* In newer firmware, the four bytes returned in data
-	 * contain rubbish, and can not be used for determining
-	 * the size of the EAX parameters. You have to discard it. */
+	// In newer firmware, the four bytes returned in data
+	// contain rubbish, and can not be used for determining
+	// the size of the EAX parameters. You have to discard it.
 	free(data);
 
-	/* 15 bytes + Status byte + 4 bytes size (again) */
+	// 15 bytes + Status byte + 4 bytes size (again)
 	data= malloc(20);
 	if (data == NULL) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
 
-	/* Read from bulk */
+	// Read from bulk
 	if ( (bread= usb_pipe_read(njb, data, actsize+5))
 		== -1 ) {
 
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(data);
 		__leave;
 		return -1;
 	} else if ( bread < actsize+5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		free(data);
 		__leave;
 		return -1;
 	}
 	
 	if ( data[0] ) {
-		NJB_ERROR(EO_BADSTATUS);
+		NJB_ERROR(njb, EO_BADSTATUS);
 		free(data);
 		__leave;
 		return -1;
@@ -1183,6 +1353,7 @@ int njb_refresh_eax(njb_t *njb, eax_t *eax)
 	__leave;
 	return 0;
 }
+*/
 
 njb_time_t *njb_get_time(njb_t *njb)
 {
@@ -1194,7 +1365,7 @@ njb_time_t *njb_get_time(njb_t *njb)
 
 	data= malloc(17);
 	if (data == NULL) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return NULL;
 	}
@@ -1202,18 +1373,18 @@ njb_time_t *njb_get_time(njb_t *njb)
 	
        	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
        			NJB_CMD_GET_TIME, 0, 0, 17, data) == -1 ) {
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
        		__leave;
        		return NULL;
        	}
        	if ( data[0] ) {
-       		NJB_ERROR(EO_BADSTATUS);
+       		NJB_ERROR(njb, EO_BADSTATUS);
 		free(data);
        		__leave;
        		return NULL;
        	}
-       	time= time_unpack(&data[1]);
+       	time = time_unpack(&data[1]);
 	
 	free(data);
 
@@ -1225,6 +1396,8 @@ int njb_set_time(njb_t *njb, njb_time_t *time)
 {
 	__dsub= "njb_set_time";
 	unsigned char *data;
+	
+	__enter;
 
        	data= time_pack(time);
 
@@ -1236,7 +1409,7 @@ int njb_set_time(njb_t *njb, njb_time_t *time)
        	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
        		NJB_CMD_SET_TIME, 0, 0, 16, data) == -1 ) {
 
-       		NJB_ERROR(EO_USBCTL);
+       		NJB_ERROR(njb, EO_USBCTL);
        	free(data);
        		__leave;
        		return -1;
@@ -1258,7 +1431,7 @@ int njb_create_playlist (njb_t *njb, char *name, u_int32_t *plid)
 
 	size= strlen(name) + 1;
 	if ( size > 0xffffffff ) {
-		NJB_ERROR(EO_TOOBIG);
+		NJB_ERROR(njb, EO_TOOBIG);
 		__leave;
 		return 0;
 	}
@@ -1270,35 +1443,35 @@ int njb_create_playlist (njb_t *njb, char *name, u_int32_t *plid)
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_CREATE_PLAYLIST, lsw, msw, 0, NULL) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	bwritten= usb_pipe_write(njb, name, size);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bwritten < size ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		__leave;
 		return -1;
 	}
 
 	bread= usb_pipe_read(njb, retdata, 5);
 	if ( bread == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bread < 5 ) {
-		NJB_ERROR(EO_RDSHORT);
+		NJB_ERROR(njb, EO_RDSHORT);
 		__leave;
 		return -1;
 	}
 
 	if ( retdata[0] ) {
-		NJB_STATUS(retdata[0]);
+		NJB_STATUS(njb,retdata[0]);
 		__leave;
 		return -1;
 	}
@@ -1323,13 +1496,13 @@ int njb_delete_playlist (njb_t *njb, u_int32_t plid)
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_DELETE_PLAYLIST, msw, lsw, 1, &data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -1351,7 +1524,7 @@ int njb_rename_playlist (njb_t *njb, u_int32_t plid, char *name)
 	
 	size= strlen(name) + 1;
 	if ( size > 0xffffffff ) {
-		NJB_ERROR(EO_TOOBIG);
+		NJB_ERROR(njb, EO_TOOBIG);
 		__leave;
 		return 0;
 	}
@@ -1362,18 +1535,18 @@ int njb_rename_playlist (njb_t *njb, u_int32_t plid, char *name)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_RENAME_PLAYLIST, 0, 0, 8, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	bwritten= usb_pipe_write(njb, name, size);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		return -1;
 	} else if ( bwritten < size ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		__leave;
 		return -1;
 	}
@@ -1396,7 +1569,7 @@ int njb_add_track_to_playlist (njb_t *njb, u_int32_t plid, u_int32_t trid)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_ADD_TRACK_TO_PLAYLIST, 0, 0, 10, data) == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -1420,7 +1593,7 @@ int njb_add_multiple_tracks_to_playlist (njb_t *njb, u_int32_t plid,
 	bsize= ntracks*6;
 	block= (unsigned char *) malloc(bsize);
 	if ( block == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
@@ -1442,7 +1615,7 @@ int njb_add_multiple_tracks_to_playlist (njb_t *njb, u_int32_t plid,
 		NJB_CMD_ADD_MULTIPLE_TRACKS_TO_PLAYLIST, 0, 0, 6, data) 
 		== -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(block);
 		__leave;
 		return -1;
@@ -1450,12 +1623,12 @@ int njb_add_multiple_tracks_to_playlist (njb_t *njb, u_int32_t plid,
 
 	bwritten= usb_pipe_write(njb, block, bsize);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		free(block);
 		__leave;
 		return -1;
 	} else if ( bwritten < bsize ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		free(block);
 		__leave;
 		return -1;
@@ -1480,13 +1653,13 @@ int njb_delete_track (njb_t *njb, u_int32_t trackid)
 
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_DELETE_TRACK, msw, lsw, 1, &data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -1508,13 +1681,13 @@ int njb_delete_datafile (njb_t *njb, u_int32_t fileid)
 
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_DELETE_DATAFILE, msw, lsw, 1, &data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
 
 	if ( data ) {
-		NJB_STATUS(data);
+		NJB_STATUS(njb,data);
 		__leave;
 		return -1;
 	}
@@ -1537,7 +1710,7 @@ int njb_play_or_queue (njb_t *njb, u_int32_t trackid, int cmd)
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER, cmd, 0, 0, 4, data)
 		 == -1 ) {
 
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -1557,7 +1730,7 @@ int njb_elapsed_time (njb_t *njb, u_int16_t *elapsed, int *change)
 	
 	if ( usb_setup(njb, UT_READ_VENDOR_OTHER,
 		NJB_CMD_ELAPSED_TIME, 0, 0, 3, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
@@ -1567,7 +1740,7 @@ int njb_elapsed_time (njb_t *njb, u_int16_t *elapsed, int *change)
 	} else if ( data[0] == 0 ) {
 		*change= 0;
 	} else {
-		NJB_STATUS(data[0]);
+		NJB_STATUS(njb,data[0]);
 		__leave;
 		return -1;
 	}
@@ -1580,19 +1753,19 @@ int njb_elapsed_time (njb_t *njb, u_int16_t *elapsed, int *change)
 
 int njb_replace_track_tag (njb_t *njb, njbttaghdr_t *tagh, void *tag)
 {
-	__dsub= "njb_replace_track_tag";
+	__dsub = "njb_replace_track_tag";
 	u_int16_t msw, lsw;
 	unsigned char *data;
 	ssize_t bwritten;
 
 	__enter;
 
-	msw= get_msw(tagh->size);
-	lsw= get_lsw(tagh->size);
+	msw = get_msw(tagh->size);
+	lsw = get_lsw(tagh->size);
 
-	data= (unsigned char *) malloc(tagh->size+4);
+	data = (unsigned char *) malloc(tagh->size+4);
 	if ( data == NULL ) {
-		NJB_ERROR(EO_NOMEM);
+		NJB_ERROR(njb, EO_NOMEM);
 		__leave;
 		return -1;
 	}
@@ -1603,21 +1776,20 @@ int njb_replace_track_tag (njb_t *njb, njbttaghdr_t *tagh, void *tag)
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_REPLACE_TRACK_TAG, lsw, msw, 0, NULL) == -1 ) {
-
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		free(data);
 		__leave;
 		return -1;
 	}
 
-	bwritten= usb_pipe_write(njb, data, tagh->size+4);
+	bwritten = usb_pipe_write(njb, data, tagh->size+4);
 	if ( bwritten == -1 ) {
-		NJB_ERROR(EO_USBBLK);
+		NJB_ERROR(njb, EO_USBBLK);
 		__leave;
 		free(data);
 		return -1;
 	} else if ( bwritten < (tagh->size+4) ) {
-		NJB_ERROR(EO_WRSHORT);
+		NJB_ERROR(njb, EO_WRSHORT);
 		free(data);
 		__leave;
 		return -1;
@@ -1639,13 +1811,13 @@ int njb_adjust_sound(njb_t *njb, u_int8_t effect, int16_t value)
 	memset(data, 0, 3);
 	data[0] = effect;
 
-	printf("Effect %d, sending value %d (%04X)\n", effect, sendvalue, sendvalue);
+	/* printf("Effect %d, sending value %d (%04X)\n", effect, sendvalue, sendvalue); */
 
 	from_16bit_to_njb1_bytes(sendvalue, &data[1]);
 
 	if ( usb_setup(njb, UT_WRITE_VENDOR_OTHER,
 		NJB_CMD_ADJUST_SOUND, 0, 0, 3, data) == -1 ) {
-		NJB_ERROR(EO_USBCTL);
+		NJB_ERROR(njb, EO_USBCTL);
 		__leave;
 		return -1;
 	}
